"""
Python Wechaty - https://github.com/wechaty/python-wechaty

Authors:    Huan LI (李卓桓) <https://github.com/huan>
            Jingjing WU (吴京京) <https://github.com/wj-Mcat>

2018-now @copyright Wechaty

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from __future__ import annotations

from enum import Enum
from dataclasses import Field, dataclass, fields, is_dataclass, field
from typing import Any, Dict, List, Optional

from wechaty_puppet.logger import get_logger
from wechaty_puppet.schemas.base import BaseDataClass

logger = get_logger("WechatyPuppet.Schemas.Event")


class ScanStatus(Enum):
    """
    scan status enum
    """
    Unknown = 0
    Cancel = 1
    Waiting = 2
    Scanned = 3
    Confirmed = 4
    Timeout = 5


@dataclass(init=False)
class EventFriendshipPayload(BaseDataClass):
    friendship_id: str


@dataclass(init=False)
class EventLoginPayload(BaseDataClass):
    contact_id: str


@dataclass(init=False)
class EventLogoutPayload(BaseDataClass):
    contact_id: str
    data: str


@dataclass(init=False)
class EventMessagePayload(BaseDataClass):
    message_id: str
    type: Optional[str] = None
    from_id: Optional[str] = None
    filename: Optional[str] = None
    text: Optional[str] = None
    timestamp: Optional[float] = None
    room_id: Optional[str] = None
    to_id: Optional[str] = None
    mention_ids: Optional[List[str]] = None


@dataclass(init=False)
class EventRoomInvitePayload(BaseDataClass):
    room_invitation_id: str


@dataclass(init=False)
class EventRoomJoinPayload(BaseDataClass):
    # TODO -> discuss name style
    invited_ids: List[str]
    inviter_id: str
    room_id: str
    timestamp: float

@dataclass(init=False)
class EventRoomLeavePayload(BaseDataClass):
    # TODO -> discuss name style
    removed_ids: List[str]
    remover_id: str
    room_id: str
    timestamp: float


@dataclass(init=False)
class EventRoomTopicPayload(BaseDataClass):
    changer_id: str
    new_topic: str
    old_topic: str
    room_id: str
    timestamp: float


@dataclass(init=False)
class EventScanPayload(BaseDataClass):
    status: ScanStatus
    qrcode: Optional[str] = None
    data: Optional[str] = None


@dataclass(init=False)
class EventDongPayload(BaseDataClass):
    data: Optional[str] = None


@dataclass(init=False)
class EventErrorPayload(BaseDataClass):
    data: str


@dataclass(init=False)
class EventReadyPayload(BaseDataClass):
    data: str


@dataclass(init=False)
class EventResetPayload(BaseDataClass):
    data: str


@dataclass(init=False)
class EventHeartbeatPayload(BaseDataClass):
    data: str



"""
Python Wechaty - https://github.com/wechaty/python-wechaty

Authors:    Huan LI (李卓桓) <https://github.com/huan>
            Jingjing WU (吴京京) <https://github.com/wj-Mcat>

2020-now @ Copyright Wechaty

Licensed under the Apache License, Version 2.0 (the 'License');
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an 'AS IS' BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from __future__ import annotations

import asyncio
import logging
from dataclasses import dataclass
from datetime import datetime
from typing import Optional, Any

from pyee import AsyncIOEventEmitter    # type: ignore

from wechaty_puppet.schemas.base import BaseDataClass

log = logging.getLogger('Watchdog')


@dataclass(init=False)
class WatchdogFood(BaseDataClass):
    """watchdog food interface"""
    timeout: int
    data: Optional[Any] = None


class Watchdog(AsyncIOEventEmitter):
    """
    watch dog based on asyncio coroutine
    """
    def __init__(self, default_timeout: int = 30, name: str = 'Bark'):
        """
        init the watch dog
        """
        super().__init__()
        self._last_feed: Optional[datetime] = None
        self._last_food: Optional[WatchdogFood] = None
        self.default_timeout: int = default_timeout
        self.name = name
        self.timer = None

    # def on(self, event, f=None) -> Watchdog:
    #     """listen for the watchdog event"""
    #     log.info('watchdog <%s> <%s> registered', event, f)
    #     super().on(event, f)
    #     return self

    async def _start_timer(self, timeout: int):
        """start the timer to record watchdog"""
        log.info('Watchdog <%s> _start_timer() setTimeout() after %d',
                 self.name, timeout)

        if self.timer is not None:
            raise ValueError('timer already exist !')

        await asyncio.sleep(timeout)
        timeout = self._last_food.timeout if self._last_food is not None else \
            self.default_timeout
        self.emit('reset', self._last_food, timeout)

    def feed(self, food: WatchdogFood):
        """feed the food to the watch dog"""
        log.debug('feed the food <%s> the watchdog', food)
        self._last_food = food
        self._last_feed = datetime.now()
        self.emit('feed', food, self._last_feed)

    async def sleep(self):
        """dog can sleep"""
        if self._last_food is not None and self._last_feed is not None:
            log.debug('sleep at <%s>, last_feed_time: <%s>, timeout: <%s>',
                      datetime.now(), self._last_feed, self.default_timeout)
            await asyncio.sleep(self._last_food.timeout)
            self.emit('sleep', self._last_food, self._last_feed)

    def starved_to_death(self) -> bool:
        """check the dog health status"""
        if self._last_feed is None:
            raise ValueError('dog has not be feed food')
        timeout = (datetime.now() - self._last_feed).seconds
        if timeout > self.default_timeout:
            self.emit('death', self._last_food, self._last_feed)
            return True
        return False



"""
file-box utils functions
"""
from copy import deepcopy
import types
import mimetypes
import os
import inspect
from typing import Any, Dict, Tuple
from urllib.parse import urlparse
import requests
import uuid


def data_url_to_base64(data_url: str) -> str:
    """
    transfer the DataUrl to base64 format

    data_url:  dataURL: `data:image/png;base64,${base64Text}`,

    return :
    """
    if ',' not in data_url:
        raise ValueError('DataUrl value is not valid, value: %s', data_url)

    return data_url.split(',')[-1]


def _extract_content_type(content_type: str) -> str:
    """extract content type"""
    if ';' in content_type:
        content_type = content_type.split(';')[0]

    if '/' in content_type:
        content_type = content_type.split('/')[-1]
    return content_type


def extract_file_name_from_url(url: str) -> Tuple[str, str]:
    """
    """
    # if we can guess the type from url
    res = requests.get(url)
    content_type = _extract_content_type(res.headers.get('Content-Type', ''))

    if mimetypes.guess_type(url)[0]:
        url_path = urlparse(url).path

        try:
            # refer to : https://stackoverflow.com/a/18727481/6894382
            file_name = os.path.basename(url_path)
            return file_name, content_type
        except TypeError as type_error:
            # return then random name of the file
            file_type, _ = mimetypes.guess_type(url)
            assert file_type is not None
            return '{}.{}'.format(str(uuid.uuid4()), file_type.split('/')[0]), \
                   content_type
    else:

        if res.status_code != 200:
            raise FileNotFoundError('can"t get the file from url <%s>' % url)

        if '?' in url:
            url = url[:url.index('?')]

        if url.endswith('/'):
            url = url[:-1]

        suffix = ''
        if not url.endswith(f'.{content_type}'):
            # if the url contains the file-name, so return it
            suffix = f'.{content_type}'

        file_name = f'{url[url.rindex("/")+1 :]}{suffix}'
        return file_name, content_type


def get_json_data(obj: object) -> Dict[str, Any]:
    """get_json_data 

    Args:
        obj (object): the instance of object

    Returns:
        Dict[str, Any]: the final json data
    """
    # 1. get all value fields
    json_data = deepcopy(obj.__dict__)

    # 2. get all properties
    all_fields = dir(obj)
    for key in all_fields:
        if key.startswith('_'):
            continue
        
        if f'_{key}' in json_data and not inspect.ismethod(getattr(obj, key)):
            json_data.pop(f'_{key}')
            json_data[key] = getattr(obj, key)
    return json_data


"""
doc
"""

# message module
from .schemas.types import (  # type: ignore

    # Message
    MessageType,
    MessagePayload,

    # Contact
    ContactGender,
    ContactType,
    ContactPayload,

    # Friendship
    FriendshipType,
    FriendshipPayload,

    # Room
    RoomPayload,
    RoomMemberPayload,

    # UrlLink

    # RoomInvitation
    RoomInvitationPayload,

    # Image
    ImageType,

    # Event
    EventType,

    # PayloadType
    PayloadType
)

from .puppet import (
    Puppet,
    PuppetOptions
)
from .file_box import FileBox

from .schemas.message import (
    MessageQueryFilter,
)

from .schemas.contact import (
    ContactQueryFilter
)

from .schemas.friendship import (
    FriendshipSearchQueryFilter
)

from .schemas.room import (
    RoomQueryFilter,
    RoomMemberQueryFilter,
)

from .schemas.url_link import UrlLinkPayload

from .schemas.mini_program import MiniProgramPayload

from .schemas.event import (
    EventScanPayload,
    ScanStatus,

    EventDongPayload,
    EventLoginPayload,
    EventReadyPayload,
    EventLogoutPayload,
    EventResetPayload,

    EventRoomTopicPayload,
    EventRoomLeavePayload,
    EventRoomJoinPayload,
    EventRoomInvitePayload,

    EventMessagePayload,
    EventHeartbeatPayload,
    EventFriendshipPayload,
    EventErrorPayload
)

from .exceptions import (
    WechatyPuppetError,
    WechatyPuppetConfigurationError,
    WechatyPuppetGrpcError,
    WechatyPuppetOperationError,
    WechatyPuppetPayloadError,
)

from .logger import get_logger

__all__ = [
    'Puppet',
    'PuppetOptions',

    'ContactGender',
    'ContactPayload',
    'ContactQueryFilter',
    'ContactType',

    'FileBox',
    'FriendshipType',
    'FriendshipSearchQueryFilter',
    'FriendshipPayload',

    'MessagePayload',
    'MessageQueryFilter',
    'MessageType',

    'UrlLinkPayload',

    'RoomQueryFilter',
    'RoomPayload',
    'RoomMemberQueryFilter',
    'RoomMemberPayload',

    'RoomInvitationPayload',

    'MiniProgramPayload',

    'EventScanPayload',
    'ScanStatus',

    'EventDongPayload',
    'EventLoginPayload',
    'EventReadyPayload',
    'EventLogoutPayload',
    'EventResetPayload',
    'EventFriendshipPayload',
    'EventHeartbeatPayload',
    'EventMessagePayload',
    'EventRoomInvitePayload',
    'EventRoomJoinPayload',
    'EventRoomLeavePayload',
    'EventRoomTopicPayload',
    'EventErrorPayload',

    'ImageType',
    'EventType',
    'PayloadType',

    'WechatyPuppetError',
    'WechatyPuppetConfigurationError',
    'WechatyPuppetGrpcError',
    'WechatyPuppetOperationError',
    'WechatyPuppetPayloadError',

    'get_logger'

]



"""
docstring
"""
from __future__ import annotations

import json
import base64
import requests
import os
import base64
from collections import defaultdict
import mimetypes

from typing import (
    Type,
    Optional,
    Union,
)

import qrcode   # type: ignore

from .utils import extract_file_name_from_url, data_url_to_base64, get_json_data

from .type import (
    FileBoxOptionsFile,
    FileBoxOptionsUrl,
    FileBoxOptionsStream,
    FileBoxOptionsBuffer,
    FileBoxOptionsQrCode,
    FileBoxOptionsBase64,
    FileBoxOptionsBase,
    Metadata, FileBoxType)

from wechaty_puppet.exceptions import WechatyPuppetConfigurationError
from wechaty_puppet.logger import get_logger


logger = get_logger("FileBox")


class FileBoxEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, bytes):
            return str(obj, encoding='utf-8')

        return json.JSONEncoder.default(self, obj)


class FileBox:
    """
    # TODO -> need to implement pipeable
    maintain the file content, which is sended by wechat
    """

    def __init__(self, options: FileBoxOptionsBase):
        logger.info('init file-box with option<%s>',  options)

        # TODO: will be deprecated after: Dec, 2022
        self._mimeType: Optional[str] = None

        self.mediaType: Optional[str] = None

        self.metadata: Metadata = defaultdict()

        self.name = options.name

        # TODO: will be deprecated after: Dec, 2022
        self.boxType: int = options.type.value
        self._type: int = self.boxType   # type: ignore

        if isinstance(options, FileBoxOptionsFile):
            self.localPath = options.path

        elif isinstance(options, FileBoxOptionsBuffer):
            self.buffer = options.buffer

        elif isinstance(options, FileBoxOptionsUrl):
            self.remoteUrl = options.url
            self.headers = options.headers

        elif isinstance(options, FileBoxOptionsStream):
            # TODO -> need to get into detail for stream sending
            self.stream = options.stream

        elif isinstance(options, FileBoxOptionsQrCode):
            self.qrCode = options.qr_code

        elif isinstance(options, FileBoxOptionsBase64):
            self.base64: bytes
            if isinstance(options.base64, str):
                self.base64 = str.encode(options.base64)
            elif isinstance(options.base64, bytes):
                self.base64 = options.base64
            else:
                raise WechatyPuppetConfigurationError(
                    f'Base64 File Data Type is invalid, str/bytes is supported')
    @property
    def mimeType(self) -> Optional[str]:
        logger.warn(
            'mimeType will be deprecated after Dec, 2022, '
            'we suggest that you should use mediaType property'
        )
        return self.mediaType
    
    @mimeType.setter
    def mimeType(self, value: str):
        logger.warn(
            'mimeType will be deprecated after Dec, 2022, '
            'we suggest that you should use mediaType property'
        )
        self.mediaType = value

    def type(self) -> FileBoxType:
        """get filebox type"""
        return FileBoxType(self._type)
    
    async def ready(self):
        """
        sync the name from remote
        """
        if self.type() == FileBoxType.Url:
            await self.sync_remote_name()

    async def sync_remote_name(self):
        """sync remote name
        refer : https://developer.mozilla.org/en-US/docs/Web/HTTP/
                Headers/Content-Disposition

                Content-Disposition: attachment; filename="cool.html"

        # wujingjing comment 2020-6-29:  according the implementation of
            file-box: https://github.com/huan/file-box/blob/e42d7207bb1cf5
                b76afb8ead6f72715f4a197b35/src/misc.ts#L66

            headers in requests package doesn't contains attribute:
                content-disposition, so we need to change the way to sync
                file-name from url type

            I find the better way to extract file name from url:
                https://stackoverflow.com/questions/18727347/how-to-extract-a-
                filename-from-a-url-append-a-word-to-it/18727481#18727481
        """
        file_box_type = self.type()
        if file_box_type != FileBoxType.Url:
            raise TypeError('type <{0}> is not remote'.format(
                file_box_type.name))

        if not hasattr(self, 'remoteUrl'):
            raise AttributeError('not have attribute url')

        url = getattr(self, 'remoteUrl')
        self.name, self.mimeType = extract_file_name_from_url(url)

    def to_json_str(self) -> str:
        """
        dump the file content to json object
        :return:
        """
        # 1. if sending voice file, so check the metadata
        if self.name.endswith('.sil'):
            if not self.metadata or not self.metadata.get('voiceLength', None):
                logger.warn(
                    'detect that you want to send voice file, '
                    'but metadata is not valid, please set it, '
                    'eg: file_box.metadata = {"voiceLength": 2000}'
                )

        json_data = get_json_data(self)

        # make type in the serialized json data
        if 'type' not in json_data:
            json_data['type'] = self.type().value

        data = json.dumps(json_data, cls=FileBoxEncoder, indent=4)
        return data

    async def to_file(self, file_path: Optional[str] = None,
                      overwrite: bool = False):
        """
        save the content to the file
        :return:
        """
        file_box_type = self.type()
        if file_box_type == FileBoxType.Url:
            if not self.mimeType or not self.name:
                await self.sync_remote_name()

        file_path = self.name if file_path is None else file_path

        if os.path.exists(file_path) and not overwrite:
            raise FileExistsError(f'FileBox.toFile(${file_path}): file exist. '
                                  f'use FileBox.toFile(${file_path}, true) to '
                                  f'force overwrite.')

        if file_box_type == FileBoxType.Buffer:
            with open(file_path, 'wb+') as f:
                f.write(self.buffer)

        elif file_box_type == FileBoxType.Url:
            with open(file_path, 'wb+') as text_io:
                # get the content of the file from url
                res = requests.get(self.remoteUrl)
                text_io.write(res.content)

        elif file_box_type == FileBoxType.QRCode:
            with open(file_path, 'wb+') as f:
                # create the qr_code image file
                img = qrcode.make(self.qrCode)
                img.get_image().save(f)

        elif file_box_type == FileBoxType.Base64:
            data = base64.b64decode(self.base64)
            with open(file_path, 'wb') as f:
                f.write(data)

        elif file_box_type == FileBoxType.Stream:
            with open(file_path, 'wb+') as f:
                f.write(self.stream)

    def to_base64(self) -> str:
        """
        transfer file-box to base64 string
        :return:
        """
        # TODO -> need to implement other data format
        return ''

    @classmethod
    def from_url(cls: Type[FileBox], url: str, name: Optional[str],
                 headers: Optional[dict] = None) -> FileBox:
        """
        create file-box from url
        """
        if name is None:
            response = requests.get(url)
            # TODO -> should get the name of the file
            name = response.content.title().decode(encoding='utf-8')
        options = FileBoxOptionsUrl(name=name, url=url, headers=headers)
        file_box: FileBox = cls(options)
        file_box.mimeType = mimetypes.guess_type(url)[0] or ''
        return file_box

    @classmethod
    def from_file(cls: Type[FileBox], path: str, name: Optional[str] = None) -> FileBox:
        """create FileBox from local file, which can be any type of file

        Examples:
            >>> file_box = FileBox.from_file('beautiful-gril.png')
            >>> # or
            >>> file_box = FileBox.from_file('hello.sil')
            >>> file_box.metadata = {'voiceLength': 2000}

        Returns:
            _description_
        """
        if not os.path.exists(path):
            raise FileNotFoundError(f'{path} file not found')
        
        if name is None:
            name = os.path.basename(path)
        
        # 1. check the audio related code
        if name.endswith('.silk') or name.endswith('.slk'):
            logger.warn('detect that you want to send voice file which should be <name>.sil pattern. So we help you rename it.')
            if name.endswith('.silk'):
                name = name.replace('.silk', '.sil')
            if name.endswith('.slk'):
                name = name.replace('.slk', '.sil')

        with open(path, 'rb') as f:
            content = base64.b64encode(f.read())

        file_box: FileBox = cls.from_base64(base64=content, name=name)

        # if sending the voice file, the mediaType must be: 'audio/silk'
        if file_box.name.endswith('.sil'):
            file_box.mediaType = 'audio/silk'
        else:
            file_box.mimeType = mimetypes.guess_type(path)[0] or ''

        return file_box

    @classmethod
    def from_stream(cls: Type[FileBox], stream: bytes, name: str) -> FileBox:
        """
        create file-box from stream

        TODO -> need to implement stream detials
        """
        options = FileBoxOptionsStream(name=name, stream=stream)
        return cls(options)

    @classmethod
    def from_buffer(cls: Type[FileBox], buffer: bytes, name: str) -> FileBox:
        """
        create file-box from buffer

        TODO -> need to implement buffer detials
        """
        options = FileBoxOptionsBuffer(name=name, buffer=buffer)
        return cls(options)

    @classmethod
    def from_base64(cls: Type[FileBox], base64: bytes, name: str = 'base64.dat') -> FileBox:
        """
        create file-box from base64 str

        refer to the file-box implementation, name field is required.

        :param base64:
            example data: data:image/png;base64,${base64Text}
        :param name: name the file name of the base64 data
        :return:
        """
        options = FileBoxOptionsBase64(base64=base64, name=name)
        return FileBox(options)

    @classmethod
    def from_data_url(cls: Type[FileBox], data_url: str, name: str) -> FileBox:
        """
        example value: dataURL: `data:image/png;base64,${base64Text}`,
        """
        return cls.from_base64(
            str.encode(data_url_to_base64(data_url)),
            name
        )

    @classmethod
    def from_qr_code(cls: Type[FileBox], qr_code: str) -> FileBox:
        """
        create file-box from base64 str
        """
        options = FileBoxOptionsQrCode(name='qrcode.png', qr_code=qr_code)
        return cls(options)

    @classmethod
    def from_json(cls: Type[FileBox], obj: Union[str, dict]) -> FileBox:
        """
        create file-box from json data

        TODO -> need to translate :
            https://github.com/huan/file-box/blob/master/src/file-box.ts#L175

        :param obj:
        :return:
        """
        if isinstance(obj, str):
            json_obj = json.loads(obj)
        else:
            json_obj = obj

        # original box_type field name is boxType
        if 'type' not in json_obj:
            raise Exception('box field must be required')
        # assert that boxType value must match the value of FileBoxType values

        if json_obj['type'] == FileBoxType.Base64.value:
            file_box = FileBox.from_base64(
                base64=json_obj['base64'],
                name=json_obj['name']
            )
        elif json_obj['type'] == FileBoxType.Url.value:
            file_box = FileBox.from_url(
                url=json_obj['remoteUrl'],
                name=json_obj['name']
            )
        elif json_obj['type'] == FileBoxType.QRCode.value:
            file_box = FileBox.from_qr_code(
                qr_code=json_obj['qrCode']
            )
        else:
            raise ValueError('unknown file_box json object %s',
                             json.dumps(json_obj))

        if 'metadata' not in json_obj:
            json_obj['metadata'] = {}
        elif not isinstance(json_obj['metadata'], dict):
            raise AttributeError('metadata field is not dict type')

        file_box.metadata = json_obj['metadata']

        return file_box



"""
Python Wechaty - https://github.com/wechaty/python-wechaty-puppet

Authors:    Huan LI (李卓桓) <https://github.com/huan>
            Jingjing WU (吴京京) <https://github.com/wj-Mcat>

2018-now @copyright Wechaty

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from __future__ import annotations

import os


def get_or_create_dir(*paths) -> str:
    """
    get or create path
    Args:
        paths: the sub path of
    Returns:
    """

    path = os.path.join(*paths)
    os.makedirs(path, exist_ok=True)
    return path


# global cache dir
CACHE_DIR: str = get_or_create_dir('.wechaty')



"""
Python Wechaty - https://github.com/wechaty/python-wechaty

Authors:    Huan LI (李卓桓) <https://github.com/huan>
            Jingjing WU (吴京京) <https://github.com/wj-Mcat>

2020-now @ Copyright Wechaty

Licensed under the Apache License, Version 2.0 (the 'License');
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an 'AS IS' BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from __future__ import annotations
import logging
import os
from typing import Optional


WECHATY_LOG_KEY = 'WECHATY_LOG'
WECHATY_LOG_FILE_KEY = 'WECHATY_LOG_FILE'
DEFAULT_LOG_LEVEL = 'INFO'


def _get_logger_level() -> str:
    """refer to : https://docs.python.org/3/library/logging.html#logging-levels

    fix: https://github.com/wechaty/python-wechaty/issues/192

    According to our Wechaty Specification <https://wechaty.js.org/docs/specs/wechaty>:
    The `WECHATY_LOG` should support the values of `silly`, `verbose`, `info`, `warn`, `silent`
    All Polyglot Wechaty should at least support the above names as an alias.
    Link to https://github.com/wechaty/wechaty/issues/2167
    """
    level_map = {
        'silly': 'CRITICAL',
        'verbose': 'INFO',
        'info': 'INFO',
        'warn': 'WARNING',
        'error': 'ERROR',
        'silent': 'NOTSET'
    }
    level: str = os.environ.get(WECHATY_LOG_KEY, DEFAULT_LOG_LEVEL).lower()
    if level not in level_map:
        raise ValueError(
            f'{WECHATY_LOG_KEY} should support the values of `silly`, '
            '`verbose`, `info`, `warn`, `error`, `silent`'
        )
    return level_map.get(level, DEFAULT_LOG_LEVEL)


def get_logger(name: Optional[str] = None, file: Optional[str] = None) -> logging.Logger:
    """get the logger object

    Args:
        name (str): the logger name
        file (Optional[str], optional): the log file name. Defaults to None.

    Examples:
        >>> logger = get_logger("WechatyPuppet")
        >>> logger = get_logger("WechatyPuppet", file="wechaty-puppet.log")
        >>> logger.info('log info ...')

    Returns:
        logging.Logger: the logger object
    """
    WECHATY_LOG = _get_logger_level()

    log_formatter = logging.Formatter(
        fmt='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    # create logger and set level to debug
    logger = logging.getLogger(name)
    logger.handlers = []
    logger.setLevel(WECHATY_LOG)
    logger.propagate = False

    # create file handler and set level to debug
    file = os.environ.get(WECHATY_LOG_FILE_KEY, file)
    if file:
        file_handler = logging.FileHandler(file, 'a', encoding='utf-8')
        file_handler.setLevel(WECHATY_LOG)
        file_handler.setFormatter(log_formatter)
        logger.addHandler(file_handler)

    # create console handler and set level to info
    console_handler = logging.StreamHandler()
    console_handler.setLevel(WECHATY_LOG)
    console_handler.setFormatter(log_formatter)
    logger.addHandler(console_handler)

    return logger
